/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Copyright (C) 2004,2019 Erik Auerswald
 * auerswal <at> unix-ag <dot> uni-kl <dot> de */

#include <SDL/SDL.h>        /* SDL */
#include <errno.h>          /* used by perror */
#include <getopt.h>         /* getopt */
#include <stdio.h>          /* puts, printf, BUFSIZ, perror */
#include <stdlib.h>         /* exit */
#include <math.h>           /* log */
#include "lsjoysticksdl.h"  /* defines etc. */

/* print usage */
void usage(char *name, FILE *f)
{
  fprintf(f,"Usage: %s [OPTION]... [NUMBER]...\n", name);
  fprintf(f,"\nOptions:\n");
  fprintf(f,"  -h, --help              print this help\n");
  fprintf(f,"  -V, --version           print version information\n");
  fprintf(f,"  -l, --long              print additional info, not just number and name\n");
  fprintf(f,"  -v, --verbose           explain what is beeing done\n");
  fprintf(f,"  -q, --quiet             suppress error messages\n");
  fprintf(f,"  -c, --count             count joysticks (no info about individual joysticks)\n");
  fprintf(f,"\nIf no numbers are specified %s tries to find joysticks,", name);
  fprintf(f,"\notherwise it probes the specified NUMBERs for joysticks.\n");
}

/* print info about a joystick */
void query_joystick(int n, int format, int number_of_joysticks)
{
  SDL_Joystick *joystick;
  if(format & VERBOSE)
    puts("querying joystick name");
  printf("joystick %*d: %s\n", (int)(log(number_of_joysticks)/log(10)), n,
      SDL_JoystickName(n));
  if(format & LONG_FORMAT)
  {
    /* open joystick to query no. of axes, buttons, trackballs and hats */
    if(format & VERBOSE)
      puts("querying  no. of joystick axes, buttons, trackballs and hats");
    joystick = SDL_JoystickOpen(n);
    printf("%*s(%d axes, %d buttons, %d trackballs, %d hats)\n",
	14+(int)(log(number_of_joysticks)/log(10)), "",
	SDL_JoystickNumAxes(joystick),
	SDL_JoystickNumButtons(joystick),
	SDL_JoystickNumBalls(joystick),
	SDL_JoystickNumHats(joystick));
    SDL_JoystickClose(joystick);
  }
}

int main(int argc, char *argv[])
{
  int i, c, n, format=SHORT_FORMAT, number_of_joysticks=0;
  char error_message[BUFSIZ];
  /* parse command line */

  /* the following is (basically) taken from the getopt(3) manpage
   * and modified for my needs */

  while (1)
  {
    int option_index = 0;
    static struct option long_options[] = {
      {"help", 0, 0, 'h'},
      {"version", 0, 0, 'V'},
      {"long", 0, 0, 'l'},
      {"quiet", 0, 0, 'q'},
      {"verbose", 0, 0, 'v'},
      {"count", 0, 0, 'c'},
      {0, 0, 0, 0}
    };

    c = getopt_long (argc, argv, "hVlqvc", long_options, &option_index);
    if (c == -1)
      break;

    switch (c)
    {
      case 'h':
	usage(argv[0],stdout);
	exit (0);
	break;

      case 'V':
	printf("%s version %s\n", argv[0], VERSION);
	exit (0);
	break;

      case 'l':
	format &= ~QUIET_FORMAT;
	format |= (SHORT_FORMAT | LONG_FORMAT);
	break;

      case 'q':
	format |= NO_ERRORS;
	break;

      case 'v':
	format |= VERBOSE;
	break;

      case 'c':
	format &= ~(SHORT_FORMAT | LONG_FORMAT);
	format |= QUIET_FORMAT;
	break;

      case '?':  /* missing argument or character not in optstring */
	usage(argv[0],stderr);
	exit (1);
	break;

      default:   /* this should not be reached */
	if(!(format & NO_ERRORS))
	  printf ("?? getopt returned character code 0%o ??\n", c);
    }
  }

  /* end of code from man page */

  /* initialize SDL */
  if(format & VERBOSE)
    puts("initializing SDL (joystick subsystem only)");
  if(SDL_Init(SDL_INIT_JOYSTICK) < 0)
  {
    if(!(format & NO_ERRORS))
    {
      snprintf(error_message, BUFSIZ, "Couldn't initialize SDL: %s",
	  SDL_GetError());
      perror(error_message);
    }
    exit(1);
  }

  /* check if joysticks are present */
  if(format & VERBOSE)
    puts("querying number of joysticks");
  number_of_joysticks = SDL_NumJoysticks();

  /* if joysticks are just to be counted we're done */
  /* otherwise query info for joysticks */
  if(format != QUIET_FORMAT)
  {
    if(format & VERBOSE)
      puts("iterating over joystick numbers");
    if(optind < argc) /* numbers specified on commandline */
    {
      if(format & VERBOSE)
	puts("getting joystick numbers from command line");
      /* iterate over commandline */
      for(i=optind; i<argc; i++)
      {
	/* get number from commandline */
	n = atoi(argv[i]);
	/* query joystick n */
	if(n < number_of_joysticks)
	{
	  if(format & VERBOSE)
	    printf("querying joystick %d\n", n);
	  query_joystick(n, format, number_of_joysticks);
	}
      }
    }
    else
    {
      /* iterate over all joysticks */
      for(i=0; i<number_of_joysticks; i++)
      {
	/* query joystick i */
	if(format & VERBOSE)
	  printf("querying joystick %d\n", i);
	query_joystick(i, format, number_of_joysticks);
      }
    }
  }

  /* output number of joysticks if appropriate */
  if(format & (QUIET_FORMAT | LONG_FORMAT))
  {
    if(number_of_joysticks != 1)
    {
      printf("%d joysticks\n", number_of_joysticks);
    }
    else
      puts("1 joystick");
  }
  /* shut down SDL */
  if(format & VERBOSE)
    puts("shutting down SDL");
  SDL_Quit();
  exit (0);
}
